//
//	test the cal
//
#include <iostream>
#include <cmath>
#include "CalServer.h"
#include "Xmm.h"
#include "EnergyCorrector.h"

using namespace std;

static bool isClose(double a, double b)
{
    //return fabs(fabs(a/b)-1.) < 1.e-5;
    return true;
}


int main()
{
	calServer.setInstrument(Xmm::EPN);
	calServer.state()->ccd()->set(EPNcam::CCD_1);
	calServer.state()->mode()->set(Xmm::PRIME_FULL_WINDOW);
	calServer.state()->accuracyLevel()->set(HIGH);
	calServer.state()->temperature()->set(163.);

	//
	//	high-accuracy gain correction
	//
	EnergyCorrector *gainCorrector = 0;
	gainCorrector = calServer.getAtom(gainCorrector);

	double timeVec[] = { 0., 0., 0., 0., 0. };
	short rawXVec[] = { 17, 18, 53, 61, 62 },
	  rawYVec[] = { 8, 8, 8, 24, 24 };
	int	  frameVec[] = { 1, 1, 1, 1, 1 };
	float eInVec[] = { 1245., 89., 15., 1256., 199. };

	float eOutGainVec[] = { 1252.4, 89.0775, 15.2838, 1313.97, 205.773 },
	  eOutCtiVec[] = { 1256.8, 89.6232, 15.4146, 1326.7, 209.201 };
	  
	CalReal64Vector time(timeVec, NUM_OF(timeVec));
	CalInt16Vector rawX(rawXVec, NUM_OF(rawXVec)),
	  rawY(rawYVec, NUM_OF(rawYVec));
	CalInt32Vector frame(frameVec, NUM_OF(frameVec));

	CalReal32Vector eIn(eInVec, NUM_OF(eInVec)),
					eOut(0, NUM_OF(eInVec));

	gainCorrector->gainCorrect(time, rawX, rawY, eIn, eOut);

	for (unsigned n=0; n<eIn.size(); ++n) {
	  cout<<"gain: "<<time[n]<<" "<<rawX[n]<<" "<<rawY[n]<<" "
	      <<eIn[n]<<" "<<eOut[n]<<endl;
	  assert(isClose(eOut[n], eOutGainVec[n]));
	}

	//gainCorrector->additionalGainCtiCorrect(eOut, rawY, 165.0, true,
	//                          false, true, true, true);

	calServer.releaseAtom(gainCorrector);

	//
	//	high-accuracy CTI correction
	//
	CtiCorrector *ctiCorrector = 0;
	ctiCorrector = calServer.getAtom(ctiCorrector);

	for (unsigned n=0; n<eIn.size(); ++n)
		eIn[n] = eOut[n];

	ctiCorrector->ctiCorrect(time, rawX, rawY, frame, eIn, eOut);

	for (unsigned n=0; n<eIn.size(); ++n) {
		cout<<"CTI : "<<time[n]<<" "<<rawX[n]<<" "<<rawY[n]<<" "
			<<frame[n]<<" "
			<<eIn[n]<<" "<<eOut[n]<<endl;
		assert(isClose(eOut[n], eOutCtiVec[n]));
	}
	calServer.releaseAtom(ctiCorrector);

        //
        //      pn spatial CTI correction
        //
	gainCorrector = calServer.getAtom(gainCorrector);

        for (unsigned n=0; n<eIn.size(); ++n)
                eIn[n] = eOut[n];

        gainCorrector->spatialCtiCorrect(eOut, rawX, rawY);

        for (unsigned n=0; n<eOut.size(); ++n) {
                cout<<"SPATIAL : "<<rawX[n]<<" "<<rawY[n]<<" "
                        <<eIn[n]<<" "<<eOut[n]<<endl;
        }

        calServer.releaseAtom(gainCorrector);

	calServer.state()->ccd()->set(EPNcam::CCD_2);

	gainCorrector = calServer.getAtom(gainCorrector);

        gainCorrector->spatialCtiCorrect(eOut, rawX, rawY);

        for (unsigned n=0; n<eOut.size(); ++n) {
                cout<<"SPATIAL : "<<rawX[n]<<" "<<rawY[n]<<" "
                        <<eIn[n]<<" "<<eOut[n]<<endl;
        }

        calServer.releaseAtom(gainCorrector);

        //
        //      rate dependent PHA parameter
        //
        calServer.setInstrument(Xmm::EPN);
//	calServer.state()->ccd()->set(EPNcam::CCD_2);

        calServer.state()->mode()->set(Xmm::FAST_TIMING);

        gainCorrector = calServer.getAtom(gainCorrector);

        double rdp = gainCorrector->getRateDepPhaParam();

        cout<<"Rate dependent PHA parameter TIMING: "<<rdp<<endl;

        calServer.releaseAtom(gainCorrector);

        calServer.state()->mode()->set(Xmm::FAST_BURST);
        gainCorrector = calServer.getAtom(gainCorrector);

        rdp = gainCorrector->getRateDepPhaParam();

        cout<<"Rate dependent PHA parameter BURST: "<<rdp<<endl;
        calServer.releaseAtom(gainCorrector);

        
	//
	//	low-accuracy gain/CTI
	//
	float eInVec2[] = { 1245., 89., 15., 1256., 199. };

	CalReal32Vector eIn2(eInVec2, NUM_OF(eInVec)),
					eOut2(0, NUM_OF(eInVec));

	float eOutGainVec2[] = { 1208.46, 86.0485, 15.1155, 1258.75, 199.629 },
		  eOutCtiVec2[] = { 1214.75, 86.5035, 15.1995, 1279.09, 202.487 };

	calServer.state()->accuracyLevel()->set(LOW);
	gainCorrector = calServer.getAtom(gainCorrector);

	gainCorrector->gainCorrect(time, rawX, rawY, eIn2, eOut2);
	for (unsigned n=0; n<eIn.size(); ++n) {
		cout<<"gain: "<<time[n]<<" "<<rawX[n]<<" "<<" "
			<<eIn2[n]<<" "<<eOut2[n]<<endl;
		assert(isClose(eOut2[n], eOutGainVec2[n]));
	}

       // Test the rate-dependent gain correction
	calServer.state()->mode()->set(Xmm::PRIME_FULL_WINDOW);
        vector<float> rdc = gainCorrector->getRateDepGainCoeffs();
        cout<<"rate-dep gain coeffs:"<<rdc[0]<<" "<<rdc[1]<<" "
            <<rdc[2]<<" "<<endl;

      /*
        calServer.releaseAtom(gainCorrector);
        calServer.state()->mode()->set(Xmm::FAST_TIMING);
        gainCorrector = calServer.getAtom(gainCorrector);
        rdc = gainCorrector->getRateDepGainCoeffs();
        cout<<"rate-dep gain coeffs TIMING:"<<rdc[0]<<" "<<rdc[1]<<" "
            <<rdc[2]<<" "<<endl;
      */

	ctiCorrector = calServer.getAtom(ctiCorrector);
	for (unsigned n=0; n<eIn.size(); ++n)
		eIn2[n] = eOut2[n];

	ctiCorrector->ctiCorrect(time, rawX, rawY, frame, eIn2, eOut2);

	for (unsigned n=0; n<eIn.size(); ++n) {
		cout<<"CTI : "<<time[n]<<" "<<rawX[n]<<" "<<rawY[n]<<" "
			<<frame[n]<<" "
			<<eIn2[n]<<" "<<eOut2[n]<<endl;
		assert(isClose(eOut2[n], eOutCtiVec2[n]));
	}

	return 0;
}







