//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	LowEnergyNoiseServer --- the LowEnergyNoiseServer class
//
//	DESCRIPTION:
//		A class 'LowEnergyNoiseServer' is defined which provides access to
//		low energy noise data in the REJECT CCF
//
//	CHANGE HISTORY:
//		RDS, 2003-11-21: 1.0.0: - first version
//
//
//	AUTHOR: R. Saxton (RDS)
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al.,
//		[2]: Calibration Data Access Handbool, C. Erd et al.
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/cal/src/LowEnergyNoiseServer.h,v 1.3 2012/03/30 12:13:47 rsaxton Exp $
//

#ifndef LOWENERGY_NOISE_SERVER_H
#define LOWENERGY_NOISE_SERVER_H

#include "Ccf.h"
#include "CalChild.h"
#include "CalAtomsBase.h"
#include "Image.h"

//
//	---( LowEnergyNoiseServer )----------------------------------------
//
class LowEnergyNoiseServer : public CalAtom,
	 public virtual CalChild<CalAtomCollection<LowEnergyNoiseServer> > {
public:
	LowEnergyNoiseServer(unsigned modeOffset, unsigned ccdOffset,
                                         const Proxy<SImage> &medianMap,
                                         const Proxy<SImage> &masterOffset,
                                         const Proxy<SImage> &masterOffClosed,
                                         vector<int8> &rawX,
					 vector<real32> &xrayloadCoeff,
                                         const vector<SImage> &noiseMap,
                                         const Table *corrValuesTable,
						CalAtomCollection<LowEnergyNoiseServer> *parent = 0);
		//
		//	PURPOSE:
		//		Instantiate LowEnergyNoiseServer object from table to
		//		CCF::EPN_REJECT table.
		//


	virtual void dump(ostream &) {}
		//
		//	PURPOSE:
		//		Dump data members to an ostream
		//
		//	ARGUMENTS:
		//		In:
		//			os:	stream to print on
		//


	static unsigned id() { return Ccf::Reject; }
		//
		//	PURPOSE:
		//		Return ID of CalAtom
		//

                                                
	const std::vector<int8> &rawxVector() const;
	const std::vector<SImage> &noiseMapVector() const;
		//
		//	PURPOSE:
		//		Return reference to rawX and associated noise map vectors
		//


	const SImage &medianMap() const;
		//
		//	PURPOSE:
		//		Return reference to median map from CCF
		//

	const SImage &masterOffset() const;
		//
		//	PURPOSE:
		//		Return reference to master Offset from CCF
		//

	const SImage &masterOffsetClosed() const;
		//
		//	PURPOSE:
		//		Return reference to master Offset CLOSED
                //              filter image from CCF
		//

const std::vector<real32> &lowEnergy() const;
const std::vector<real32> &highEnergy() const;
const std::vector<real32> &xrayLoadCoeff() const;
const std::vector<int16> &correctionValue() const;
		//
		//	PURPOSE:
		//		Return reference to energy range and correction value vectors
		//

	virtual ~LowEnergyNoiseServer();
		//
		//	PURPOSE:
		//		Destroy object constructed with above constructor(s).
		//

private:
	//	data members
	unsigned			_modeOffset;
  unsigned _ccdOffset;
	Proxy<SImage>		_medianMap;
	Proxy<SImage>		_masterOffset;
	Proxy<SImage>		_masterOffClosed;
  std::vector<int8> _rawX;
  std::vector<SImage> _noiseMap;
  
	std::vector<real32>	_hlow,_hhigh;
	std::vector<real32>     _xrayloadCoeff;	
  std::vector<int16>  _corrValue;
}; // LowEnergyNoiseServer

#endif
