//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	OmGrismDataServer.cc --- implementation of OmGrismDataServer class
//
//	DESCRIPTION:
//
//	CHANGE HISTORY:
//		UL/RDS, 2003-10-21: 1.0.0: - initial version
//
//	AUTHOR:
//		U. Lammers (UL), R. Saxton, Tue Oct 21 10:16:48 GMT 2003
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al.,
//		[2]: Calibration Access and Data Handbook, XMM instrument team
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/cal/src/OmGrismDataServer.cc,v 1.3 2009/07/06 14:25:19 aibarra Exp $
//
#include <iostream>

#include "OmGrismDataServer.h"
#include "CalOalUtility.h"
#include "stringutil.h"

using namespace std;

//
//	---( OmGrismDataServer> ) ---------------------------------------
//
OmGrismDataServer::OmGrismDataServer(const CcfConstituentTable &flux,
									 const CcfConstituentTable &wavelength,
						CalAtomCollection<OmGrismDataServer> *parent)
		: CalChild<CalAtomCollection<OmGrismDataServer> >(parent)
{
	CalOalUtility::fillVector(flux.block(), "WAVELENGTH", _isf.wavelength);
	CalOalUtility::fillVector(flux.block(), "ISF", _isf.data);
	CalOalUtility::fillVector(flux.block(), "ISF_error", _isf.error);

	CalOalUtility::fillVector(wavelength.block(), "WAVELENGTH", _wavelength);
} // OmGrismDataServer::OmGrismDataServer()


void OmGrismDataServer::dump(ostream &os)
{
	os<<"OmGrismDataServer{\n";
	os<<"ISF data\n"
	  <<"lamdba data error\n";
    unsigned N = _isf.wavelength.size();
	for (unsigned n=0; n<N; n++)
		os<<_isf.wavelength[n]<<' '<<_isf.data[n]<<' '<<_isf.error[n]<<endl;
	os<<"wavelength:\n";
	N = _wavelength.size();
	for (unsigned n=0; n<N; ++n)
		os<<_wavelength[n]<<endl;
	os<<"}";
} // OmGrismDataServer::dump()


OmGrismDataServer::~OmGrismDataServer()
{
} // OmGrismDataServer::~OmGrismDataServer()


//
//	---( CalAtomCollection<OmGrismDataServer> ) ----------------------
//
/* AI gcc4 porting
 * too few template-parameter-lists => template<>
 */
template<>
CalAtomCollection<OmGrismDataServer>::CalAtomCollection(
								const XmmInstrument *instrument,
								const Ccf *ccf,
								Xmm::Filter filter,
								AccuracyLevel accuracy)
{
	_filter		= filter;
	_accuracy	= accuracy;
	_nCCDs		= instrument->nCcds();
	_nNodes		= instrument->nNodes();
	_nEntries	= 1;
	OmGrismDataServer *null = 0;
	_collection = new vector<OmGrismDataServer *>(_nEntries, null);
	OmGrismDataServer *&gds = (*_collection)[0];

	//
	//	access data tables
	//
	CifEntry *e = ccf->entry(instrument, Ccf::GrismData);
	DataSet *dataSet = e->open();
	CcfConstituentTable ccfTabIsf(dataSet, "FLUX_" +
										 ::uppercase(Xmm::toString(_filter)),1),
					    ccfTabLambda(dataSet, "WAVELENGTH_" +
										 ::uppercase(Xmm::toString(_filter)),1);

   /* The CCF structure remained the same between algoid=0 and 1
      but the calling program can only handle the latter. Here, we
      print a warning if an algoid=0 file is encountered */
    if (ccfTabIsf.algoid() == 0 )
       errstr<<"Unsupported ALGOID "<<ccfTabIsf.algoid()
             <<". You are using an obsolete calibration file "
             <<dataSet->qualifiedName()<<" known to produce "
             <<"wrong results. Please upgrade to the latest CCF"
             <<Sas::warning("UnsupportedAlgoId");

	gds = new OmGrismDataServer(ccfTabIsf, ccfTabLambda, this);

	dataSetServer->close(dataSet);
} // CalAtomCollection<OmGrismDataServer>::CalAtomCollection()

/* AI gcc4 porting
 * too few template-parameter-lists => template<>
 */
template<>
OmGrismDataServer *CalAtomCollection<OmGrismDataServer>::select(
					Ccd,
					Node,
					unsigned) const
{
	return (*_collection)[0];
} // CalAtomCollection<OmGrismDataServer>::select
