//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	ScreenThresholdServer --- the ScreenThresholdServer class
//
//	DESCRIPTION:
//		A class 'ScreenThresholdServer' is defined which provides access to the
//		RGS template background data
//
//	CHANGE HISTORY:
//		RDS, 2009-04-12: 1.0.0: - first version
//
//	AUTHOR:
//     R.D. Saxton
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al.,
//		[2]: Calibration Data Access Handbook, C. Erd et al.
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/cal/src/ScreenThresholdServer.h,v 1.2 2009/07/06 14:25:19 aibarra Exp $
//

#ifndef SCREENTHRESH_SERVER_H
#define SCREENTHRESH_SERVER_H

#include "Dal.h"
#include "Ccf.h"
#include "CalChild.h"
#include "CalAtomsBase.h"
#include "slatec++.h"
#include "Proxy.h"


//
//	---( MOS screen threshold server class )---------------------------------------------------------------
//
class ScreenThresholdServer : public CalAtom,
     public virtual CalChild<CalAtomCollection<ScreenThresholdServer> > {
public:
      ScreenThresholdServer(const float smin, const float smax,
                            const float hmin, const float hmax,
                            const float rmin, const float rmax,
                            const string &thresh,
                          CalAtomCollection<ScreenThresholdServer> *parent = 0);
        //
        //  PURPOSE:
        //      Instantiate ScreenThresholdServer object from table ScreenThresh
        //

          virtual ~ScreenThresholdServer();
        //
        //  PURPOSE:
        //      Destruct ScreenThresholdServer object
        //

	  virtual void dump(ostream &) {}
        //
        //  PURPOSE:
        //      Dump data members to an ostream
        //
        //  ARGUMENTS:
        //      In:
        //          os: stream to print on
        //

          static unsigned id() { return Ccf::ScreenThresh; }
        //
        //  PURPOSE:
        //      Return ID of CalAtom
        //

    // Return the soft, hard and rate band ranges
     /* AI gcc4 porting
      * type qualifiers ignored on function return type
      * => remove const
      */
      float softbandmin() const;
      float softbandmax() const;
      float hardbandmin() const;
      float hardbandmax() const;
      float ratebandmin() const;
      float ratebandmax() const;

    // Return the threshold
      string threshold();


protected:
     // Calculate the position of the relevant ScreenThreshold object
    /* AI gcc4 porting
      * type qualifiers ignored on function return type
      * => remove const
      */
	 unsigned index() const;

 	private:
    float _smin, _smax;
    float _hmin, _hmax;
    float _rmin, _rmax;
    string _thresh;
    //string &_thresh;

}; // ScreenThreshServer



#endif
