//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	SpecQualityServer.h --- miscellaneous Cal atoms
//
//	DESCRIPTION:
//
//	CHANGE HISTORY:
//		RDS, 2009-04-13: 1.0: - original
//
//	AUTHOR:
//		R. Saxton, 2009-04-13
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al., 
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/cal/src/SpecQualityServer.h,v 1.6 2010/04/12 14:45:25 rsaxton Exp $
//

#ifndef SPEC_QUAL_H
#define SPEC_QUAL_H

#include "Ccf.h"
#include "CalAtomsBase.h"

#include <vector>

//
//	---( mode parameters )------------------------------------------------
//
class SpecQualityServer : public CalAtom {
public:
	friend class CalAtomCollection<SpecQualityServer>;

	//	default constructor - does nothing
	SpecQualityServer() {}

	static unsigned id() { return Ccf::SpecQual; }

	short nGroups() const { return _ngroups; }
        std::vector<short> ranges() const { return _ranges; }
	float gainSlopeMin() const { return _gslope_min; }
	float gainSlopeMax() const { return _gslope_max; }
	float gainOffsetMin() const { return _offset_min; }
	float gainOffsetMax() const { return _offset_max; }
        std::vector<float> resolution_energy() const { return _res_energy; }
        std::vector<float> resolution() const { return _resolution; }

	~SpecQualityServer() {}

        std::vector<short> beta_ranges() const { return _beta_ranges; }
        std::vector<short> lambda_ranges() const { return _ranges; }

        int lambda_nchan() const { return _lambda_nchan; }
        int beta_nchan() const { return _beta_nchan; }

        // Methods to return ranges in lambda and beta units
        /*
        std::vector<double> lambdaRanges() const
        {
          std::vector<double> lamRange;

          double deltaLambda = (_lambda_max - _lambda_min) / (_lambda_nchan-1);
          for (unsigned n=0; n<_ranges.size(); ++n) {
              lamRange.push_back( (_ranges[n]-1) *deltaLambda + _lambda_min);
          }
          return lamRange;
        }

        std::vector<double> betaRanges() const
        {
          std::vector<double> betRange;

          double deltaBeta = (_beta_max - _beta_min) / (_beta_nchan-1);
          for (unsigned n=0; n<_bchan_ranges.size(); ++n) {
              betRange.push_back( (_bchan_ranges[n]-1) *deltaBeta + _beta_min);
          }
          return betRange;
        }
        */

private:
        std::vector<short> _ranges;  // Channel ranges for EPIC and RGS/Lambda
        std::vector<short> _beta_ranges; // CHannel ranges for BETA spectra
        short _ngroups;
	float _gslope_min,
		  _gslope_max,
		  _offset_min,
		  _offset_max;
//        double _lambda_min, 
//               _lambda_max,
//	       _beta_min,
//	       _beta_max;
        int    _lambda_nchan,
               _beta_nchan;
        std::vector<float> _res_energy;
        std::vector<float> _resolution;

};


#endif

