//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	TimeJumpTolServer --- class to access the time jump tolerance data
//			      which is a function of revolution
//
//	DESCRIPTION:
//		Instances of class TimeJumpTolServer give access to the event pattern
//		library.
//
//	CHANGE HISTORY:
//		UL, 2000-05-18: 1.0.0: - first incarnation (from CalAtomsEmos)
//
//	AUTHOR:
//		U. Lammers, Mon Oct  6 15:32:57 WET DST 1997
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al., 
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/cal/src/TimeJumpTolServer.h,v 1.1 2012/12/18 14:50:24 rsaxton Exp $
//

#ifndef TIMEJUMP_TOL_SERVER_H
#define TIMEJUMP_TOL_SERVER_H

#include "Ccf.h"
#include "CalChild.h"
#include "CalAtomsBase.h"
#include "CalAtomsMisc.h"

//
//	---( TimeJumpTolServer )----------------------------------------------------
//
class TimeJumpTolServer : public CalAtom,
	 public virtual CalChild<CalAtomCollection<TimeJumpTolServer> > {
public:
	TimeJumpTolServer(vector<int> v, vector<int> rev,
                          CalAtomCollection<TimeJumpTolServer> *parent = 0);
		//
		//	PURPOSE:
		//		Instantiate TimeJumpTolServer object from 
		//              a mode-dependent tolerance vector


	virtual void dump(std::ostream &) {}
		//
		//	PURPOSE:
		//		Dump data members to an ostream
		//
		//	ARGUMENTS:
		//		In:
		//			os:	stream to print on
		//


	static unsigned id() { return Ccf::TimeJumpTol; }
		//
		//	PURPOSE:
		//		Return ID of CalAtom
		//


	virtual int getJumpTolerance(int rev) const;
		//
		//	PURPOSE:
		//		Return the jump tolerance for a given revolution
		//


	virtual ~TimeJumpTolServer();
		//
		//	PURPOSE:
		//		Destroy object constructed with above constructor(s).
		//

private:
	vector<int>  _v;  // The tolerance vector as fn of revolution
	vector<int>  _rev;  // The revolutions for the tolerance calibration
};

#endif


